#!/usr/bin/perl

use strict;
use File::Copy;
use Sys::Hostname;

#############################################################################
# _get_dhcpd2
#
# returns 1 if dhcp server is enabled on this interface, 0 if disabled
#############################################################################
sub _get_dhcpd2
{
    my $eth = shift;
    $eth ||= 'eth1';

    my $file = "/etc/dhcpd.conf";
    my $content = _cat($file);
    my $dhcpd2 = 0;
    for (split/\n+/,$content)
    {
        if(/^subnet (\d+\.\d+\.\d+\.\d)/)
        {
            if ($1 ne '0.0.0.0')
            {
                $dhcpd2 = 1;
                last;
            }
        }
    }
    return $dhcpd2;
}


##############################################################################
# _cat
#
# uses perl's open() to open a file and return its contents
##############################################################################
sub _cat
{
    my $file = shift;
    my $contents;
    open IN, "<$file" || die ("Could not open $file: $!");
    while(<IN>)
    {
	$contents .= $_;
    }
    close IN;
    return $contents;
}



##############################################################################
# enable_dhcp_client
##############################################################################
sub enable_dhcp_client {
	my($file) = '/etc/sysconfig/network';
	my %setting = %{ $_[0] };
	my $eth = $setting{eth};
	$eth ||= 'eth0';
	chomp(my$hostname=`hostname`);

	# Take the GATEWAY= line out of /etc/sysconfig/network
	open(FILE, ">$file") || die "Unable to open $file: $!";
	print FILE "NETWORKING=yes\nHOSTNAME=${hostname}\n";
	close(FILE);

	# Install the default ifcfg-ethN file
	$file = "/etc/sysconfig/network-scripts/ifcfg-$eth";
	open(FILE, ">$file") || die "Unable to open $file: $!";
	print FILE "DEVICE=$eth\nBOOTPROTO=dhcp\nONBOOT=yes\nTYPE=Ethernet\nDHCP_HOSTNAME=pbxtra0000\n";
	close(FILE);

	net_start();
}

##############################################################################
# enable_static_ip: (ip,subnet_mask,gateway,network,broadcast,dns1,dns2)
##############################################################################
sub enable_static_ip {
	my %setting = %{ $_[0] };
	chomp(my$hostname=`hostname`);

	$setting{eth} ||= 'eth0';
	$setting{ip} ||= _get_ip_address($setting{eth});
	$setting{sn} ||= _get_netmask($setting{eth});
	$setting{gw} ||= _get_default_route();
	$setting{nw} ||= _get_network_addr($setting{ip},$setting{eth});
	$setting{bc} ||= _get_broadcast_addr($setting{ip},$setting{eth});

	my $file = '/etc/sysconfig/network';

	# Take the GATEWAY= line out of /etc/sysconfig/network
	open(FILE, ">$file") || die "Unable to open $file: $!";
	print FILE "NETWORKING=yes\nHOSTNAME=${hostname}\n";
	print FILE "GATEWAY=$setting{gw}\n";
	close(FILE);

	# Install the default ifcfg-ethN file
	$file = "/etc/sysconfig/network-scripts/ifcfg-$setting{eth}";
	open(FILE, ">$file") || die "Unable to open $file: $!";
	print FILE "DEVICE=$setting{eth}\n";
	print FILE "BOOTPROTO=static\n";
	print FILE "ONBOOT=yes\n";
	print FILE "IPADDR=$setting{ip}\n"; # if $setting{dhcpd2};
	print FILE "NETMASK=$setting{sn}\n"; # if $setting{dhcpd2};
	print FILE "TYPE=Ethernet\n";
	print FILE "DHCP_HOSTNAME=pbxtra0000\n";
	print FILE "NETWORK=$setting{nw}\n";
	print FILE "BROADCAST=$setting{bc}\n";
	close(FILE);

}

sub turn_on_dhcpd2
{
    my $dhcpd_file = '/etc/dhcpd.conf';
    my $tmp = '/tmp/dhcpd.conf.'.time();
    my $internal = '10.116.116.1'; # the ip we ALWAYS set for eth1
    my $tftp_ip = $internal;
    my $domain_name = hostname();
    $domain_name =~ s/(pbxtra|trixbox)(\d+)/s$2\.$1\.fonality\.com/;
    my $type = $1;
    my $sid=$2;

    chomp $domain_name;

    open IN, "<$dhcpd_file" || die "Can't open $dhcpd_file: $!";
    open TEMP, ">$tmp" || die "Can't open temp file: $!";
    while(<IN>)
    {
	print TEMP $_;
    }
    close IN;

    unless(_get_dhcpd2())
    {
	while(<DATA>)
	{
	    # switch ip address in that file to $internal ip.
	    s/(\d+\.\d+\.\d+\.\d+)\"/$internal\"/;
	    # switch the host name
	    s/SID/$domain_name/;
	    s/tftp-server-name "(\d+\.\d+\.\d+\.\d+)"/tftp-server-name "$tftp_ip"/;
	    print TEMP "$_";
	}
    }    
    close TEMP;
    copy($tmp,$dhcpd_file);
    unlink($tmp);

    # make sure the ip of the 2nd interface is $internal
    my $file = '/etc/sysconfig/network-scripts/ifcfg-eth1';
    $tmp = '/tmp/ifcfg-eth1.'.time();
    open(ETH1, $file) || die "Can't open $file: $!";
    open(TEMP, ">$tmp") || die "Can't open temp file: $!";
    while(<ETH1>) {
	if(/^ONBOOT=/) {
	    print TEMP "DEVICE=eth1\n";
	    print TEMP "BOOTPROTO=static\n";
	    print TEMP "ONBOOT=yes\n";
	    print TEMP "IPADDR=$internal\n";
	    print TEMP "NETMASK=255.255.252.0\n";
	    print TEMP "TYPE=Ethernet\n";
	    print TEMP "DHCP_HOSTNAME=pbxtra0000\n";
	    print TEMP "NETWORK=10.116.116.0\n";
	    print TEMP "BROADCAST=10.116.119.255\n";
	}
	elsif(/^(?:IPADDR|DEVICE|BOOTPROTO|ONBOOT|NETMASK|TYPE|DHCP_HOSTNAME|NETWORK|BROADCAST)/) {
	    next;
	}
	else {
	    print TEMP $_;
	}
    }
    close(ETH1);
    close(TEMP);
    copy($tmp, $file) || die $!;
    unlink($tmp);

    # changing the bind view of named.conf
    $file = '/etc/named.conf';
    $tmp = '/tmp/named.conf.'.time();
    open(NAMED, $file) || die "Can't open $file: $!";
    open(TEMP, ">$tmp") || die "Cano't open temp file: $!";
    my $inloop = 0;
    while(<NAMED>)
    {
	if(/zone\s+"s$sid\.$type\.fonality\.com"\s+IN\s?\{/)
	{
	    $inloop = 1;
	}
	elsif(/^\};$/ && $inloop)
	{
	    $inloop = 0;
	}
	elsif(!$inloop)
	{
	    print TEMP $_;
	}
    }
    close NAMED;
	
    # add the needed lines time
    print TEMP << "EOF";
// START ETH1 CONFS

acl "phone_hosts" {
    10.116.116.0/22;
};

view "internal" {
    match-clients { phone_hosts; };

    zone "s${sid}.${type}.fonality.com" IN {
	type master;
	file "db.s${sid}.eth1.${type}.fonality.com";
	allow-update { none; };
    };
};

view "external" {
    match-clients { !phone_hosts; any; };

    zone "s${sid}.${type}.fonality.com" IN {
	type master;
	file "db.s${sid}.${type}.fonality.com";
	allow-update {none;};
    };
};

// END ETH1 CONFS
EOF
;
    close TEMP;
    copy($tmp,$file) || die $!;
    unlink($tmp);
    
    # make the db.sSID.eth1.TYPE.fonality.com file
    my @time = localtime();
    my $serial = sprintf("%04d%02d%02d%02d", $time[5] + 1900, $time[4] + 1, $time[3], $time[2]);

    $file = "/var/named/chroot/var/named/db.s${sid}.eth1.${type}.fonality.com";
    open ZONE, ">$file" || die "Can't open $file: $!";
    print ZONE << "EOF";
\$ORIGIN .
\$TTL 3600      ; 1 hour
s$sid.$type.fonality.com       IN SOA  s$sid.$type.fonality.com. support.fonality.com. (
     $serial ; serial
     10800           ; refresh (3 hours)
     1800            ; retry (30 minutes)
     604800          ; expire (1 week)
     3600            ; minimum (1 hour)
                )
                NS              s$sid.$type.fonality.com.
                A               $internal
                MX      10      s$sid.$type.fonality.com.
EOF
;
close(ZONE);

    # bring the interface back up
    # ifdown eth1 && sleep 2 && ifup eth1
    system("/sbin/ifdown eth1 && /bin/sleep 2 && /sbin/ifup eth1");

    # don't forget to restart the daemon!
    system('/etc/init.d/dhcpd restart');
    system('/etc/init.d/named reload');
}


sub turn_off_dhcpd2
{
    my $dhcpd_file = '/etc/dhcpd.conf';
    my $tmp = '/tmp/dhcpd.conf.'.time();
    my $domain_name = hostname();
    $domain_name =~ s/(pbxtra|trixbox)(\d+)/s$2\.$1\.fonality\.com/;
    my $type = $1;
    my $sid=$2;

    open IN, "<$dhcpd_file" || die "Can't open $dhcpd_file: $!";
    open TEMP, ">$tmp" || die "Can't open temp file: $!";
    my $in_subnet = 0;
    while(<IN>)
    {
	if(/subnet (\d+\.\d+\.\d+\.\d)/)
	{
	    if( $1 eq '0.0.0.0')
	    {
		print TEMP $_;
	    }
	    else
	    {
		$in_subnet = 1;
	    }
	}
	elsif($in_subnet && /\}/)
	{
	    $in_subnet = 0;
	}
	elsif($in_subnet)
	{
	    next;
	}
	else
	{
	    print TEMP $_;
	}
    }
    close TEMP;
    copy($tmp,$dhcpd_file);
    unlink($tmp);

    # remove the stuff we added when we enabled dhcp
    my $inloop = 0;
    my $file = '/etc/named.conf';
    my $tmp = '/tmp/named.conf.'.time();
    open (NAMED, $file) || die "Can't open $file: $!";
    open (TEMP, ">$tmp") || die "Can't open temp file: $!";
    while(<NAMED>)
    {
	if(/START ETH1 CONFS/)
	{
	    $inloop = 1;
	}
	elsif(/END ETH1 CONFS/ && $inloop)
	{
	    $inloop = 0;
	}
	elsif(!$inloop)
	{
	    print TEMP $_;
	}
    }
    close NAMED;

    # add in the named stuff for eth0
    print TEMP << "EOF";
zone "s$sid.$type.fonality.com" IN {
    type master;
    file "db.s$sid.$type.fonality.com";
    allow-update { none; };
};
EOF
;

    close(TEMP);
    copy($tmp,$file);
    unlink($tmp);


    # don't forget to restart the daemon!
    system('/etc/init.d/dhcpd restart');
    system('/etc/init.d/named reload');
}

##############################################################################
# _get_network_addr
##############################################################################
sub _get_network_addr {
	my($ip) = shift(@_);
	my($eth) = shift(@_);
	$eth ||= 'eth0';
	my($a,$b,$c,$d) = split(/\./, $ip);
	my($nm) = _get_netmask($eth);
	if($nm eq '255.255.255.0') {
		return("$a.$b.$c.0");
	}
	else
	{
	    return _calc_network($ip,$nm);
	}
}


##############################################################################
# _calc_network
##############################################################################
sub _calc_network  {
    return( _int2dot( _dot2int($_[0]) & _dot2int($_[1])));
}

############################################################################
# _calc_broadcast
############################################################################
sub _calc_broadcast  {
    return ( _int2dot(( _dot2int($_[0]) & _dot2int($_[1])) + (~ _dot2int($_[1]))));
}

############################################################################
# _dot2int
# changes dot notation into integer
############################################################################
sub _dot2int  {
    return (unpack("N",pack("C4",split"[.]",$_[0])));
}

############################################################################
# _int2dot
# changes intgeter into dot notation
############################################################################
sub _int2dot  {
    return (join(".",unpack("C4",pack("N",$_[0]))));
}


##############################################################################
# _get_broadcast_addr
##############################################################################
sub _get_broadcast_addr {
    my($ip) = shift;
    my($eth) = shift;
    $eth ||= 'eth0';
	my($a,$b,$c,$d) = split(/\./, $ip);
	my($nm) = _get_netmask();
	if($nm eq '255.255.255.0') {
		return("$a.$b.$c.255");
	}
	else {
		return(_get_broadcast($ip,$eth));
	}
}

##############################################################################
# _get_broadcast
##############################################################################
sub _get_broadcast {
	my($ip) = shift || die "Must pass IP address to _get_broadcast()";
	my($eth) = shift;
	$eth ||= 'eth0';
	my($bc) ='0.0.0.0';
	my($cmd) = "/sbin/ifconfig $eth";
	my(@out) = `$cmd`;
	foreach my $line (@out) {
		if($line =~ /Bcast:(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) {
			$bc = $1;
		}
	}
	return($bc);
}


##############################################################################
# _get_nameservers
##############################################################################
sub _get_nameservers {
	my(@nameservers);
	my($file) = '/etc/resolv.conf';

	if(-f $file) {
		open(RESOLV, $file) || die "Can't open $file: $!";
		while(<RESOLV>) {
			if(/nameserver\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) {
				push(@nameservers, $1);
			}
		}
		close(RESOLV);
	}

	unless(scalar(@nameservers)) { # No nameservers found
		push(@nameservers, '4.2.2.6');
		push(@nameservers, '4.2.2.5');
	}

	return(\@nameservers);
}

##############################################################################
# check_static_nameservers
##############################################################################
sub check_static_nameservers {
	my($rpc) = shift;
	my(@nameservers);
	my($file) = '/etc/dhclient-eth0.conf';

	my $content = _cat($file);
	for( split /\n/, $content ) {
	    if(/^supersede/) {
		return 'STATIC';
	    }
	    elsif(/^prepend/)
	    {
		return 'STATIC';
	    }
	}

	return 0;
}

#############################################################################
# has_2nd_network_card
#############################################################################
sub has_2nd_network_card {
    my $cmd = '/sbin/ifconfig eth1';
    
    my $response = `$cmd`;
    
    if ($response && ($response !~ /Device not found/))
    {
	
	return 1;
    }
    else
    {
	return undef;
    }
}

##############################################################################
# _set_dnameservers (set dns back to DYANMIC)
##############################################################################
sub _set_dnameservers {
    
    my $no_netstart = shift || 0;
    
    # update /etc/dhclient-eth0.conf to get rid of lines
    # preventing dhclient from updating resolv.conf
    my $file = '/etc/dhclient-eth0.conf';
    my $tmp = '/tmp/dhclient-eth0.conf'.time();

    open FILE, $file || die "Can't open $file: $!";
    open TEMP, ">$tmp" || die "Can't open $tmp: $!";

    while(<FILE>)
    {
	# if we have a prepend or supersede line, skip it
	next if (/^prepend domain-name-servers/);
	next if (/^supersede domain-name/);
	    
	print TEMP $_;
    }
    close FILE;
    close TEMP;
	
    copy($tmp,$file) || die "Can't copy to $file: $!";
    unlink($tmp);

    unless ($no_netstart)
    {
	net_start();
    }
}


##############################################################################
# _set_nameservers
##############################################################################
sub _set_nameservers {
	my($ns1) = shift(@_) || '127.0.0.1';
	my($ns2) = shift(@_);
	my($file) = '/etc/resolv.conf';
	my($tmp) = '/tmp/resolv.conf'.time();
	my($hostname) = `/bin/hostname`;
	chomp($hostname);

	open TEMP, ">$tmp" || die "Can't open $tmp:  $!";
	print TEMP "search $hostname.fonality.com\n";
	print TEMP "nameserver $ns1\n";
	print TEMP "nameserver $ns2\n" if ($ns2);
	close TEMP;

	# copy the new resolv.conf over the old one
	copy($tmp,$file) || die "Can't copy to $file: $!";
	unlink($tmp);	


	# update /etc/dhclient-eth0.conf so that dhclient doesn't
	# overwrite /etc/resolv.conf
	$file = '/etc/dhclient-eth0.conf';
	$tmp = '/tmp/dhclient-eth0.conf'.time();
	
	open TEMP, ">$tmp" || die "Can't open $tmp: $!";

	# we want the ns1 to be the last line b/c it'll get
	# prepended first that way
	print TEMP "supersede domain-name \"$hostname.fonality.com\";\n";
	if($ns2)
	{
	    print TEMP "prepend domain-name-servers $ns2;\n";
	}
	print TEMP "prepend domain-name-servers $ns1;\n";
	close TEMP;
	
	copy($tmp,$file) || die "Can't copy to $file: $!";
	unlink($tmp);
}

##############################################################################
# _get_default_route
##############################################################################
sub _get_default_route {
	my($gw);
	my($cmd) = "/bin/netstat -rn | /bin/grep \"^0.0.0.0\"";
	my($route_info) = `$cmd`;
	if($route_info =~ /^0.0.0.0\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) {
		$gw = $1;
	}

	# If the network is down, get the passed in IP or get the one from eth0 and calculate
	# (and by calculate i mean "assume x.y.z.1")
	if (! defined $gw or ! $gw)
	{
		$gw = join '.', (split /\./, (@_ && _is_ip($_[0]) ? shift : _get_ip_address()))[0..2], 1;
	}


	return($gw);
}

##############################################################################
# _get_ip_address
##############################################################################
sub _get_ip_address {
	my($ip) = '0.0.0.0';
	my($eth) = shift;
	$eth ||= 'eth0';
	my($cmd) = "/sbin/ifconfig $eth";
	my(@out) = `$cmd`;
	foreach my $line (@out) {
		if($line =~ /inet addr:(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) {
			$ip = $1;
		}
	}
	return($ip);
}

##############################################################################
# _get_netmask
##############################################################################
sub _get_netmask {
	my $eth = shift;
	$eth ||= 'eth0';

	my($nm) = '255.255.255.0';
	my($cmd) = "/sbin/ifconfig $eth";
	my(@out) = `$cmd`;

	foreach my $line (@out) {
		if($line =~ /Mask:(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/) {
			$nm = $1;
		}
	}
	return($nm);
}


##############################################################################
# _get_bootproto
##############################################################################
sub _get_bootproto {
	my($dhcp) = 0;
	my($file) = '/etc/sysconfig/network-scripts/ifcfg-eth0';
	open(FILE, $file) || die "Unable to open file: $file: $!";
	while(<FILE>) {
		$dhcp = 1 if(/BOOTPROTO=dhcp/);
	}
	close(FILE);
	return("DHCP") if($dhcp == 1);
	return("STATIC");
}

##############################################################################
# _is_ip: Return true if the passed text is an ip address
##############################################################################
sub _is_ip {
	my($text) = shift(@_);
	return(1) if($text =~ /^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$/);
	return(0);
}

##############################################################################
# _is_available: Verify that these settings will not result in an IP conflict
##############################################################################
sub _is_available {
	my($host) = shift(@_);
	my($p) = Net::Ping->new();

	# There was an answer (bad)
	if($p->ping($host,1))
	{
		$p->close();
		return(0);
	}

	# There was no answer (good)
	$p->close();
	return(1);
}

##############################################################################
# net_stop: stop the net only if we're running locally
##############################################################################
sub net_stop
{
	system("service vpn stop");
}

##############################################################################
# net_start: start the net only if we're running locally; otherwise reboot
##############################################################################
sub net_start
{
	system("/etc/init.d/network restart");
	system("service vpn start");
	system("/usr/local/bin/apachectl startssl");
}

return 1;


__DATA__
subnet 10.116.116.0 netmask 255.255.252.0 {
    next-server 10.116.116.1;
    option subnet-mask 255.255.252.0;
    option broadcast-address 10.116.119.255;
    option domain-name "SID";
    option tftp-server-name "10.116.116.1";
    option domain-name-servers 10.116.116.1;
    option routers 10.116.116.1;
    option ntp-servers 10.116.116.1;
    range dynamic-bootp 10.116.116.10 10.116.119.254;
}
