#!/usr/bin/perl
#
# Copyright 2007, Fonality Inc. ALL RIGHTS RESERVED.

chdir("/var/adm/bin/");

use lib qw( /usr/local/rrdtool-1.2.15/lib/perl );
use lib qw( /usr/local/rrdtool-1.2.13/lib/perl );
use RRDs;

use Sys::Hostname;
use IO::Socket;
use Data::Dumper;
use strict;
use Benchmark;

use Time::HiRes qw(usleep);

use Mail::Sender;

# local host
my $hostname = hostname;
my ($server_id) = $hostname =~ /(\d+)/;

# global sock
my $sock;

# global stuff
my $siplastm;	# sip last modified time
my $zaplastm;	# zap last modified time
my %nottrunks;	# these are not trunks, these are sip/zap phones
my @linkconf;    # /etc/asterisk/fonality/link.conf data
my $linkmod = 0; # modified time of link.conf


my $time;

my $DEBUG = 0;

require('server_stats_alerts.pl');

my %stat;
my $conf;

# create the tables
my $start = time - 2;
my $dst   = "AVERAGE";
my $DB2   = "/var/lib/asterisk/call_stats.rrd";
my ($obj, $stats, $cpu, $mem, $rec);



if(! -e $DB2){
		RRDs::create(
		$DB2, "--step=1",
		"--start=$start",
		"DS:calls:GAUGE:2:0:1536",
		"DS:conf:GAUGE:2:0:1536",
		"DS:rec:GAUGE:2:0:1536",
		"DS:trunks:GAUGE:2:0:1536",
		"RRA:$dst:0.5:1:3600",
		"RRA:$dst:0.5:360:60",
		"RRA:$dst:0.5:1440:60",
		"RRA:$dst:0.5:10080:60",
		"RRA:$dst:0.5:43200:60",
	) or print STDERR "Cannot create rrd (" . RRDs::error .")";
}


# make connection for asterisk remote manager
mgr_connect();

my $last_alert;
my($pid_file) = '/var/run/asterisk.pid';
my $pid;

my $alarm_hash;

my $last_pidm;



# run loop twice a second
my $num;

my $pid = fork();
if ($pid)
{
	print "Running in BG ($pid)\n";
	exit(0);
}

close(STDIN);
close(STDERR);
close(STDOUT);

while(usleep(800000))
{
	my $newtime = time();

	# if last time < now
	if ($time != $newtime)
	{
		$time = $newtime;
		$conf = get_conf($conf);
		
		my $data = get_channel_data();
		
		# to find out number of recordings
		my @recs = glob("/var/spool/asterisk/monitor/*-in-*.wav");
		
		# add hud recordings
		push(@recs, glob("/var/spool/asterisk/monitor/*-in.wav"));		
		$rec = 0;

		# we only want recordings that have been updated in the last 10 seconds
		foreach my $file (@recs)
		{
			my ($time) = (stat($file))[9];

			# if it's less than 10 secs old, it's a current recording
			if (time() < $time + 10)
			{
				$rec++;
			}
		}
	
		if 
			(($data->{'trunks'} >= $conf->{trunks}->{value} && defined($conf->{trunks}->{value})) 
			|| 
			(($data->{'trunks'} > 10) && $DEBUG))	
		{
			send_alarm($conf->{trunks}->{email},"Trunks",$data->{'trunks'}, 'trunks', $conf);

		}

#		print "$time:".$data->{'channels'}.":".$data->{'conferences'}.":".$rec.":".$data->{'trunks'}."\n";

		# update the "calls" RRD table
		unless (RRDs::update($DB2, "$time:".$data->{'channels'}.":".$data->{'conferences'}.":".$rec.":".$data->{'trunks'}))
		{
			print STDERR "Cannot update rrd (" . RRDs::error . "$!)";
		}

		# Make sure the linked servers are up.
		my @linked_issues = linked_server_issues($conf);
		if (scalar(@linked_issues))
		{
			foreach my $s (@linked_issues)
			{
				my $key = "link" . $s;

				# We need a slightly different e-mail to be sent out.
				my $oldmsg = $conf->{message};
				$conf->{message} = "Server [SERVER_ID] can't reach linked server [USAGE]. "
					. "[NUMALERTS] alert(s) have been sent in the past hour. Disable "
					. "this message in Admin Panel->Status->alerts.";
				
				send_alarm($conf->{$key}->{email},"Linked Servers",$s,$key,$conf);

				# Restore the original message.
				$conf->{message} = $oldmsg;
			}
		}
	}
}
close($sock);


# make usable data out of "show channels"
sub get_channel_data
{
	my $conf = 0;
	my $conf2 = 0;
	my $channels = 0;
	my $channels2 = 0;
	my $trunks = 0;
	my $trunks2 = 0;

	# is this also a trunk?
	populate_trunks();

	ssend("Action: command\nCommand: show channels concise\n");
	while (<$sock>)
	{
#		(/^(\S+)\s+\((\S+)?\s+(?:(\S+)\s+)?(\d+)\s*\)\s+(\S+)\s+(\S.*)\s+(\S+)\s*$/);
		my ($chan, $context, $ext, $pri, $state, $appl, $data, $cid) = split(":");

		if ($appl eq "MeetMe")
		{
			$conf++;
		}

		if ($chan =~ /^\w+\//)
		{
			$channels++;

			# is this a trunk (not a not-trunk)?
			if ($chan =~ /^((?:IAX2?|Zap|SIP)\/[a-zA-Z\d]+)/ && !$nottrunks{$1})
			{
				$trunks++;
			}
		}
		elsif (/^--END COMMAND--/)
		{
			last;
		}
	}


	# weird asterisk dropping channels bug
	# so well poll 2 times

	select(undef, undef, undef, .02);
	ssend("Action: command\nCommand: show channels concise\n");
	while (<$sock>)
	{	
		my ($chan, $context, $ext, $pri, $state, $appl, $data, $cid) = split(/:/);

		if ($appl eq "MeetMe")
		{
			$conf2++;
		}

		if ($chan =~ /^\w+\//)
		{
			$channels2++;

			# is this a trunk (not a not-trunk)?
			if ($chan =~ /^((?:IAX2?|Zap|SIP)\/[a-zA-Z\d]+)/ && !$nottrunks{$1})
			{
				$trunks2++;
			}
		}
		elsif (/^--END COMMAND--/)
		{
			last;
		}
	}

	if ($channels2 > $channels)
	{
		$channels = $channels2;
		$conf = $conf2;
		$trunks = $trunks2;
	}

	return {channels=>$channels, conferences=>$conf, trunks=>$trunks};
}


# remore manager simple send function
sub ssend
{
	my $data = shift;
	my $noconnect = shift;
	my $odata = $data;

	$data .= "\n";
	$data =~ s/\n/\r\n/gs;

	eval
	{
		$sock->send($data);
	};
	if ($@ && !$noconnect)
	{
		mgr_connect();
		ssend($odata, 1);
	}
}

# Remote manager send function that waits for a response.
sub srequest
{
	my $data = shift;
	my $noconnect = shift;
	my $odata = $data;

	$data .= "\n";
	$data =~ s/\n/\r\n/gs;

	eval
	{
		$sock->send ($data);
	};
	if ($@ && !$noconnect)
	{
		mgr_connect();
		return srequest($odata, 1);
	}

	# Now get a response.
	my $response;
	my $timeout = 20;
	eval
	{
		local $SIG{ALRM} = sub {
			die "Timed out!";
		};
		alarm $timeout;
		while (<$sock>)
		{
			$response .= $_;
			if ($_ =~ /^\-\-END COMMAND\-\-/)
			{
				last;
			}
		}
	};

	return $response;
}

sub mgr_connect
{
	$sock = IO::Socket::INET->new("localhost:5038");
	unless ($sock)
	{
		print STDERR "Can't connect to localhost:5038\n";#...waiting 10 secs\n";
		return;
#		sleep 10;
#		mgr_connect();
	}
	else
	{
		<$sock>;
		ssend("Action: login\nUsername: local_mgr\nSecret: 0chanc3yo\n");
	}
	return $sock;
}


# create a hash of sip and zap devices that are NOT trunks.
# when we get a channel list, we'll be able to determine
# whether this call is a trunk by seeing if it's not in the
# not-trunks list. we do this because some trunks are not
# defined in sip.conf
sub populate_trunks
{
	# has sip.conf been changed
	my ($lastm) = (stat("/etc/asterisk/sip.conf"))[9];
	if ($lastm > $siplastm)
	{
		# empty previous sip trunks
		foreach (keys %nottrunks)
		{
			if (/^SIP/)
			{
				delete $nottrunks{$_};
			}
		}

		# read in new not-trunks
		open(SIP, "</etc/asterisk/sip.conf");
		while (<SIP>)
		{
			if (/^\[([A-F\d]{12}|NOSUPPORT\d+|SOFTPHONE\d+)\]/)
			{
				$nottrunks{"SIP/$1"} = 1;
			}
		}
		close(SIP);
		$siplastm = $lastm;
	}

	# check for zap changes
	($lastm) = (stat("/etc/asterisk/zapata.conf"))[9];
	if ($lastm > $zaplastm)
	{
		# empty previous zap trunks
		foreach (keys %nottrunks)
		{
			if (/^Zap/)
			{
				delete $nottrunks{$_};
			}
		}

		$nottrunks{"Zap/pseudo"} = 1; # conference channel
		open(ZAP, "</etc/zaptel.conf");
		while (<ZAP>)
		{
			if (/^(?:fxo[lkg]s|bchan)\s*=(.*)$/)
			{
				my $chans = $1;
				$chans =~ s/;.*$//;
				$chans =~ s/\s//g;

				foreach (split(",", $chans))
				{
					if (/^(\d+)-(\d+)$/)
					{
						map { $nottrunks{"Zap/$_"} = 1 } $1 .. $2;
					}
					else
					{
						$nottrunks{"Zap/$_"} = 1;
					}
				}
			}
		}
		close(ZAP);

		my $signalling;
		open(ZAP, "</etc/asterisk/zapata.conf");
		while (<ZAP>)
		{
			if (/^signalling\s*=\s*(\S+)/)
			{
				$signalling = $1;
			}
			elsif (/^channel\s*=>\s*(.*)$/)
			{
				next unless $signalling eq "pri_cpe";

				# if we're pri_cpe, we want to get rid of these trunks
				my $chans = $1;
				$chans =~ s/;.*$//;
				$chans =~ s/\s//g;

				foreach (split(",", $chans))
				{
					if (/^(\d+)-(\d+)$/)
					{
						map { delete $nottrunks{"Zap/$_"} } $1 .. $2;
					}
					else
					{
						delete $nottrunks{"Zap/$_"};
					}
				}
			}
		}
		close(ZAP);

		$zaplastm = $lastm;
	}
}

sub linked_server_issues
{
	my $conf = shift;

	# See if we need to re-read the linked server config.
	my ($mtime) = (stat("/etc/asterisk/fonality/link.conf"))[9];
	if ($mtime > $linkmod)
	{
		$linkmod = $mtime;
		open (READ, "/etc/asterisk/fonality/link.conf");
		@linkconf = <READ>;
		close (READ);
		chomp @linkconf;
	}
	
	my $peers = srequest("Action: command\nCommand: iax2 show peers\n");

	# Test the results.
	my @lines = split(/\n/, $peers);
	my $results = {};
	my @issues = ();
	foreach my $ln (@lines)
	{
		my ($name,$host,$s,$mask,$port,$status) = split(/\s+/, $ln);
		next unless $name =~ /^\w\d+\w$/;
		my ($c,$sid,$if) = $name =~ /^(\w)(\d+)(\w)$/;
		next if $sid == $server_id;

		# Only continue if we're actively monitoring this connection.
		my $key = "link$sid";
		next unless $conf->{$key}->{enable};
		next unless length $conf->{$key}->{email} > 0;

		# Make sure this server is still linked with us.
		my $stillActive = 0;
		foreach my $line (@linkconf)
		{
			if ($line =~ /$sid/)
			{
				$stillActive = 1;
				last;
			}
		}
		next unless $stillActive;

		if ($if eq 'x')
		{
			# External IF
			$results->{$sid}->{ex} = $status;
		}
		else
		{
			# Internal IF
			$results->{$sid}->{in} = $status;
		}
	}

	# Evaluate the results. If we have a successful connection to this
	# server on either interface, we should be good. If the remote servers
	# have trouble connecting back to us... the instance of this script
	# running on that server should report the alert...
	foreach my $sid (keys %{$results})
	{
		if ($results->{$sid}->{ex} =~ /OK/ || $results->{$sid}->{in} =~ /OK/)
		{
			# We're good.
		}
		else
		{
			# There's a problem here.
			push (@issues,$sid);
		}
	}
	
	return @issues;
}
